var restposten = require('..');
var memory = require('restposten-memory');
var JaySchema = require('jayschema');

// load the JSON Schemas
var authorSchema = require('./author.schema.json');
var bookSchema = require('./book.schema.json');

// set the validator
restposten.validator = new JaySchema;

// registering schemas is only required if you have schemas that reference each
// other
restposten.validator.register(authorSchema);

// connect to the database
memory.connect(function(err, engine) {
  // register the engine
  restposten.engine = engine;

  var Author = restposten.define('author', authorSchema);

  var Book = restposten.define('book', {
    "properties" : {
      "id" : {
        "type" : "string"
      },
      "author" : {
        "type" : "string",
        "links" : [ {
          "rel" : "full",
          "href" : "author/{@}"
        } ]
      }
    }
  });

  // create an author
  Author.create({
    "_id" : "pvorb"
  }, function(err, author) {
    if (err)
      throw err;
    console.log(author._id, 'created');

    // create a book
    Book.create({
      "_id" : "some-book",
      "author" : "pvorb"
    }, function(err, book) {
      if (err)
        throw err;

      console.log(book._id, 'created');

      // get the author of this book
      book.getAuthor(function(err, author) {
        if (err)
          throw err;

        console.log(book._id, 'written by', author._id);
      });

      // get all books from this author
      author.getBooks(function(err, books) {
        if (err)
          throw err;

        console.log(author._id, 'wrote:');
        books.forEach(function(book) {
          console.log('  *', book._id);
        })
      });
    });
  });
});
